@@ -1,23 +1,10 @@
 /*
   This file is part of the kcalcore library.
 
-  Copyright (c) 2001-2003 Cornelius Schumacher <schumacher@kde.org>
-  Copyright (C) 2003-2004 Reinhold Kainhofer <reinhold@kainhofer.com>
+  SPDX-FileCopyrightText: 2001-2003 Cornelius Schumacher <schumacher@kde.org>
+  SPDX-FileCopyrightText: 2003-2004 Reinhold Kainhofer <reinhold@kainhofer.com>
 
-  This library is free software; you can redistribute it and/or
-  modify it under the terms of the GNU Library General Public
-  License as published by the Free Software Foundation; either
-  version 2 of the License, or (at your option) any later version.
-
-  This library is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  Library General Public License for more details.
-
-  You should have received a copy of the GNU Library General Public License
-  along with this library; see the file COPYING.LIB.  If not, write to
-  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-  Boston, MA 02110-1301, USA.
+  SPDX-License-Identifier: LGPL-2.0-or-later
 */
 /**
   @file
@@ -31,22 +18,25 @@
 #ifndef KCALCORE_INCIDENCE_H
 #define KCALCORE_INCIDENCE_H
 
-#include "kcalendarcore_export.h"
 #include "alarm.h"
 #include "attachment.h"
+#include "conference.h"
 #include "incidencebase.h"
+#include "kcalendarcore_export.h"
 #include "recurrence.h"
 
 #include <QMetaType>
 
 //@cond PRIVATE
 // Value used to signal invalid/unset latitude or longitude.
-#define INVALID_LATLON 255.0 //krazy:exclude=defines (part of the API)
+#define INVALID_LATLON 255.0 // krazy:exclude=defines (part of the API)
 //@endcond
 
 namespace KCalendarCore
 {
 
+class IncidencePrivate;
+
 /**
   @brief
   Provides the abstract base class common to non-FreeBusy (Events, To-dos,
@@ -66,14 +56,13 @@ namespace KCalendarCore
   and Incidence contains all additional properties that are common to
   Events, Todos and Journals, but are not allowed for FreeBusy entries.
 */
-class KCALENDARCORE_EXPORT Incidence
-    : public IncidenceBase, public Recurrence::RecurrenceObserver
+class KCALENDARCORE_EXPORT Incidence : public IncidenceBase, public Recurrence::RecurrenceObserver
 {
     Q_GADGET
     Q_PROPERTY(QString description READ description WRITE setDescription)
     Q_PROPERTY(QString summary READ summary WRITE setSummary)
     Q_PROPERTY(QString location READ location WRITE setLocation)
-    Q_PROPERTY(bool hasGeo READ hasGeo WRITE setHasGeo)
+    Q_PROPERTY(bool hasGeo READ hasGeo)
     Q_PROPERTY(float geoLatitude READ geoLatitude WRITE setGeoLatitude)
     Q_PROPERTY(float geoLongitude READ geoLongitude WRITE setGeoLongitude)
     Q_PROPERTY(QStringList categories READ categories WRITE setCategories)
@@ -81,24 +70,24 @@ class KCALENDARCORE_EXPORT Incidence
     Q_PROPERTY(QDateTime created READ created WRITE setCreated)
     Q_PROPERTY(KCalendarCore::Incidence::Secrecy secrecy READ secrecy WRITE setSecrecy)
     Q_PROPERTY(KCalendarCore::Incidence::Status status READ status WRITE setStatus)
-    Q_PROPERTY(QVariantList attachments READ attachmentsVariant)
+    Q_PROPERTY(QList<KCalendarCore::Attachment> attachments READ attachments)
+    Q_PROPERTY(QList<KCalendarCore::Conference> conferences READ conferences)
 public:
-
     /**
       The different types of overall incidence status or confirmation.
       The meaning is specific to the incidence type in context.
     */
     enum Status {
-        StatusNone,           /**< No status */
-        StatusTentative,      /**< event is tentative */
-        StatusConfirmed,      /**< event is definite */
-        StatusCompleted,      /**< to-do completed */
-        StatusNeedsAction,    /**< to-do needs action */
-        StatusCanceled,       /**< event or to-do canceled; journal removed */
-        StatusInProcess,      /**< to-do in process */
-        StatusDraft,          /**< journal is draft */
-        StatusFinal,          /**< journal is final */
-        StatusX               /**< a non-standard status string */
+        StatusNone, /**< No status */
+        StatusTentative, /**< event is tentative */
+        StatusConfirmed, /**< event is definite */
+        StatusCompleted, /**< to-do completed */
+        StatusNeedsAction, /**< to-do needs action */
+        StatusCanceled, /**< event or to-do canceled; journal removed */
+        StatusInProcess, /**< to-do in process */
+        StatusDraft, /**< journal is draft */
+        StatusFinal, /**< journal is final */
+        StatusX, /**< a non-standard status string */
     };
     Q_ENUM(Status)
 
@@ -106,9 +95,9 @@ public:
       The different types of incidence access classifications.
     */
     enum Secrecy {
-        SecrecyPublic,      /**< Not secret (default) */
-        SecrecyPrivate,     /**< Secret to the owner */
-        SecrecyConfidential /**< Secret to the owner and some others */
+        SecrecyPublic, /**< Not secret (default) */
+        SecrecyPrivate, /**< Secret to the owner */
+        SecrecyConfidential, /**< Secret to the owner and some others */
     };
     Q_ENUM(Secrecy)
 
@@ -117,9 +106,9 @@ public:
        Only RelTypeParent is supported for now.
     */
     enum RelType {
-        RelTypeParent,  /**< The related incidence is a parent. */
-        RelTypeChild,   /**< The related incidence is a child. */
-        RelTypeSibling  /**< The related incidence is a peer. */
+        RelTypeParent, /**< The related incidence is a parent. */
+        RelTypeChild, /**< The related incidence is a child. */
+        RelTypeSibling, /**< The related incidence is a peer. */
     };
 
     /**
@@ -130,12 +119,17 @@ public:
     /**
       List of incidences.
     */
-    typedef QVector<Ptr> List;
+    typedef QList<Ptr> List;
+
+    Incidence() = delete;
 
     /**
-      Constructs an empty incidence.*
+      Constructs an empty incidence.
+      @param p (non-null) a Private data object provided by the instantiated
+      class (Event, Todo, Journal, FreeBusy). It passes ownership of the object
+      to IncidenceBase.
     */
-    Incidence();
+    Incidence(IncidencePrivate *p);
 
     /**
       Destroys an incidence.
@@ -410,9 +404,25 @@ public:
     */
     Q_REQUIRED_RESULT QString relatedTo(RelType relType = RelTypeParent) const;
 
-// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-// %%%%%  Convenience wrappers for property handling
-// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+    /**
+      Set the incidence color, as added in RFC7986.
+
+      @param colorName a named color as defined in CSS3 color name, see
+       https://www.w3.org/TR/css-color-3/#svg-color.
+      @since: 5.76
+     */
+    void setColor(const QString &colorName);
+
+    /**
+      Returns the color, if any is defined, for this incidence.
+
+      @since: 5.76
+     */
+    Q_REQUIRED_RESULT QString color() const;
+
+    // %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+    // %%%%%  Convenience wrappers for property handling
+    // %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     /**
        Returns true if the alternative (=text/html) description is
        available.
@@ -434,9 +444,9 @@ public:
     */
     Q_REQUIRED_RESULT QString altDescription() const;
 
-// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-// %%%%%  Recurrence-related methods
-// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+    // %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+    // %%%%%  Recurrence-related methods
+    // %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
     /**
       Returns the recurrence rule associated with this incidence. If there is
@@ -480,8 +490,7 @@ public:
       given date; an empty list if the incidence does not overlap with the
       date at all.
     */
-    virtual QList<QDateTime> startDateTimesForDate(const QDate &date,
-                                                   const QTimeZone &timeZone) const;
+    virtual QList<QDateTime> startDateTimesForDate(const QDate &date, const QTimeZone &timeZone) const;
 
     /**
       Calculates the start date/time for all recurrences that happen at the
@@ -505,9 +514,9 @@ public:
     */
     Q_REQUIRED_RESULT virtual QDateTime endDateForStart(const QDateTime &startDt) const;
 
-// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-// %%%%%  Attachment-related methods
-// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+    // %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+    // %%%%%  Attachment-related methods
+    // %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
     /**
       Adds an attachment to the incidence.
@@ -545,9 +554,9 @@ public:
     */
     void clearAttachments();
 
-// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-// %%%%%  Secrecy and Status methods
-// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+    // %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+    // %%%%%  Secrecy and Status methods
+    // %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
     /**
       Sets the incidence #Secrecy.
@@ -565,7 +574,9 @@ public:
 
     /**
       Sets the incidence status to a standard #Status value.
-      Note that StatusX cannot be specified.
+      Events, Todos, and Journals each have a different set of
+      valid statuses.  Note that StatusX cannot be specified.
+      Invalid statuses are logged and ignored.
 
       @param status is the incidence #Status to set.
       @see status(), setCustomStatus().
@@ -593,9 +604,9 @@ public:
     */
     Q_REQUIRED_RESULT Status status() const;
 
-// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-// %%%%%  Other methods
-// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+    // %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+    // %%%%%  Other methods
+    // %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
     /**
       Sets a list of incidence resources. (Note: resources in this context
@@ -635,35 +646,32 @@ public:
     Q_REQUIRED_RESULT bool hasGeo() const;
 
     /**
-      Sets if the incidence has geo data.
-      @param hasGeo true if incidence has geo data, otherwise false
-      @see hasGeo(), geoLatitude(), geoLongitude().
-    */
-    void setHasGeo(bool hasGeo);
-
-    /**
-      Set the incidences geoLatitude.
-      @param geolatitude is the incidence geolatitude to set
+      Set the incidence's geoLatitude.
+      @param geolatitude is the incidence geolatitude to set; a value between -90.0 and 90.0,
+      or INVALID_LATLON (or NaN, which is treated as INVALID_LATLON).
       @see geoLatitude().
     */
     void setGeoLatitude(float geolatitude);
 
     /**
-      Returns the incidence geoLatidude.
+      Returns the incidence's geoLatitude as a value between -90.0 and 90.0 or INVALID_LATLON.
+      If either of geoLatitude() and geoLongitude() are INVALID_LATLON, then both are, and hasGeo() is false.
       @return incidences geolatitude value
       @see setGeoLatitude().
     */
     Q_REQUIRED_RESULT float geoLatitude() const;
 
     /**
-      Set the incidencesgeoLongitude.
-      @param geolongitude is the incidence geolongitude to set
+      Set the incidence's geoLongitude.
+      @param geolongitude is the incidence geolongitude to set; a value between -180.0 and 180.0,
+      or INVALID_LATLON (or NaN, which is treated as INVALID_LATLON).
       @see geoLongitude().
     */
     void setGeoLongitude(float geolongitude);
 
     /**
-      Returns the incidence geoLongitude.
+      Returns the incidence's geoLongitude as a value between -180.0 and 180.0 or INVALID_LATLON.
+      If either of geoLatitude() and geoLongitude() are INVALID_LATLON, then both are, and hasGeo() is false.
       @return incidences geolongitude value
       @see setGeoLongitude().
     */
@@ -708,9 +716,9 @@ public:
     */
     Q_REQUIRED_RESULT bool thisAndFuture() const;
 
-// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-// %%%%%  Alarm-related methods
-// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+    // %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+    // %%%%%  Alarm-related methods
+    // %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
     /**
       Returns a list of all incidence alarms.
@@ -749,9 +757,41 @@ public:
     */
     Q_REQUIRED_RESULT bool hasEnabledAlarms() const;
 
-// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-// %%%%%  Other methods
-// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+    // %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+    // %%%%% Conferences-related method
+    // %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
+    /**
+     * Returns list of all incidence conferencing methods.
+     * @since 5.77
+     */
+    Q_REQUIRED_RESULT Conference::List conferences() const;
+
+    /**
+     * Replaces all conferences in the incidence with given @p conferences
+     *
+     * @param conferences New conferences to store in the incidence.
+     * @since 5.77
+     */
+    void setConferences(const Conference::List &conferences);
+
+    /**
+     * Adds a conference to the incidence.
+     *
+     * @param conferene A conference to add.
+     * @since 5.77
+     */
+    void addConference(const Conference &conference);
+
+    /**
+     * Removes all conferences from the incidence.
+     * @since 5.77
+     */
+    void clearConferences();
+
+    // %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+    // %%%%%  Other methods
+    // %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
     /**
       Set the incidence scheduling ID. Do _not_ use with journals.
@@ -777,8 +817,7 @@ public:
              is ignored.
       @see schedulingID().
     */
-    void setSchedulingID(const QString &sid,
-                         const QString &uid = QString());
+    void setSchedulingID(const QString &sid, const QString &uid = QString());
 
     /**
       Returns the incidence scheduling ID. Do _not_ use with journals.
@@ -823,12 +862,15 @@ public:
     Q_REQUIRED_RESULT static QStringList mimeTypes();
 
 protected:
+    Incidence(const Incidence &) = delete;
 
     /**
-      Copy constructor.
       @param other is the incidence to copy.
+      @param p (non-null) a Private data object provided by the instantiated
+      class (Event, Todo, Journal, FreeBusy). It passes ownership of the object
+      to IncidenceBase.
     */
-    Incidence(const Incidence &other);
+    Incidence(const Incidence &other, IncidencePrivate *p);
 
     /**
       Compares this with Incidence @p incidence for equality.
@@ -852,14 +894,10 @@ private:
       virtual function assign.
       @param other is another Incidence object to assign to this one.
      */
-    Incidence &operator=(const Incidence &other);
+    Incidence &operator=(const Incidence &other) = delete;
 
-    Q_DECL_HIDDEN QVariantList attachmentsVariant() const;
-
-    //@cond PRIVATE
-    class Private;
-    Private *const d;
-    //@endcond
+protected:
+    Q_DECLARE_PRIVATE(Incidence)
 };
 
 }
@@ -872,7 +910,7 @@ inline uint qHash(const QSharedPointer<KCalendarCore::Incidence> &key)
 //@endcond
 
 //@cond PRIVATE
-Q_DECLARE_TYPEINFO(KCalendarCore::Incidence::Ptr, Q_MOVABLE_TYPE);
+Q_DECLARE_TYPEINFO(KCalendarCore::Incidence::Ptr, Q_RELOCATABLE_TYPE);
 Q_DECLARE_METATYPE(KCalendarCore::Incidence *)
 //@endcond
 
